@@ -235,6 +235,21 @@ class Accessible {
     return false;
   }
 
+  /**
+   * Return the closest common ancestor of `this` and `aAcc`, potentially
+   * including `this`. That is, if `aAcc` is `this` or a descendant, this method
+   * will return `this`.
+   */
+  const Accessible* GetClosestCommonInclusiveAncestor(
+      const Accessible* aAcc) const;
+
+  Accessible* GetClosestCommonInclusiveAncestor(Accessible* aAcc) {
+    const Accessible* common =
+        const_cast<const Accessible*>(this)->GetClosestCommonInclusiveAncestor(
+            aAcc);
+    return const_cast<Accessible*>(common);
+  }
+
   /**
    * Used by ChildAtPoint() method to get direct or deepest child at point.
    */
@@ -389,6 +404,9 @@ class Accessible {
 
   virtual Maybe<int32_t> GetIntARIAAttr(nsAtom* aAttrName) const = 0;
 
+  virtual bool GetStringARIAAttr(nsAtom* aAttrName,
+                                 nsAString& aAttrValue) const = 0;
+
   /**
    * Get the relation of the given type.
    */
@@ -427,11 +445,6 @@ class Accessible {
    */
   virtual nsAtom* TagName() const = 0;
 
-  /**
-   * Return input `type` attribute
-   */
-  virtual already_AddRefed<nsAtom> InputType() const = 0;
-
   /**
    * Return a landmark role if applied.
    */
@@ -585,6 +598,8 @@ class Accessible {
 
   bool IsHTMLRadioButton() const { return mType == eHTMLRadioButtonType; }
 
+  bool IsHTMLSpinner() const { return mType == eHTMLSpinnerType; }
+
   bool IsHTMLTable() const { return mType == eHTMLTableType; }
   bool IsHTMLTableCell() const { return mType == eHTMLTableCellType; }
   bool IsHTMLTableRow() const { return mType == eHTMLTableRowType; }
@@ -629,8 +644,6 @@ class Accessible {
 
   bool IsDateTimeField() const { return mType == eHTMLDateTimeFieldType; }
 
-  bool IsSearchbox() const;
-
   virtual bool HasNumericValue() const = 0;
 
   /**
@@ -712,7 +725,16 @@ class Accessible {
   /**
    * Return the localized string for the given key.
    */
-  static void TranslateString(const nsString& aKey, nsAString& aStringOut);
+  static void TranslateString(const nsString& aKey, nsAString& aStringOut,
+                              const nsTArray<nsString>& aParams = {});
+
+  /*
+   * Return calculated group level based on accessible hierarchy.
+   *
+   * @param aFast  [in] Don't climb up tree. Calculate level from aria and
+   *                    roles.
+   */
+  virtual int32_t GetLevel(bool aFast) const;
 
  protected:
   // Some abstracted group utility methods.
@@ -733,14 +755,6 @@ class Accessible {
    */
   virtual AccGroupInfo* GetOrCreateGroupInfo() = 0;
 
-  /*
-   * Return calculated group level based on accessible hierarchy.
-   *
-   * @param aFast  [in] Don't climb up tree. Calculate level from aria and
-   *                    roles.
-   */
-  virtual int32_t GetLevel(bool aFast) const;
-
   /**
    * Calculate position in group and group size ('posinset' and 'setsize') based
    * on accessible hierarchy.
